#!/home/perl/bin/perl


#
# Threading code (ithreads)
# 
use threads;
use threads::shared;

# 
# Local modules
#
use lib "./lib/perl5"; 

#
# SOAP modules
#
#use SOAP::Lite +trace => all;
use SOAP::Transport::HTTP::Daemon::CookieDaemon;
use SOAP::Transport::Handler;

# make unbuffered
$|=1;					

# SOAP Server
# -----------
my $daemon;
my $thread;

my $soap_server = sub {

   print "Starting SOAP server\n";
  
   $daemon = eval { new SOAP::Transport::HTTP::Daemon::CookieDaemon(
                      LocalHost     => '127.0.0.1',
                      LocalPort     => 8080,
                      Listen        => 5, 
                      Reuse         => 1) };   
                    
   if ($@) {
      # If we restart the user agent process quickly after a crash the port 
      # will still be blocked by the operating system and we won't be able 
      # to start the daemon. Other than the port being in use I can't see
      # why we're going to end up here.
      my $error = "$@";
      chomp($error);
      return "FatalError: $error";
   };
   
   # print some info
   print "SOAP server at " . $daemon->url() ,"\n";     
    
   # handlers directory
   my $handler = "SOAP::Transport::Handler";
   
   # defined handlers for the server
   $daemon->dispatch_with({ 'urn:/demo' => $handler });
   $daemon->objects_by_reference( $handler );
      
   # handle it!
   print "Starting handlers for urn:/demo ...\n";
   $daemon->handle;

};


# Spawn the SOAP server thread
print("Spawning SOAP Server thread...\n");
$listener_thread = threads->create( $soap_server );

$status = $listener_thread->join() if defined $listener_thread;
print "Threads joined: $status\n";

exit;

   
