package Package::Handler;

# Basic handler class for SOAP requests for the User Agent. It also
# acts as a container class for SOAP::User class which handles authentication.

use strict;
use subs qw( new set_user ping echo );

#
# Threading code (ithreads)
# 
use threads;
use threads::shared;

#
# General modules
#
use SOAP::Lite;
use Digest::MD5 'md5_hex';
use Time::localtime;

# 
# SOAP modules
#
use SOAP::User;

# ==========================================================================
# U S E R   A U T H E N T I C A T I O N
# ==========================================================================

sub new {
  my ( $class, $user, $passwd ) = @_;
  
  my $self = bless {}, $class;
  
  print "Package::Handler constructor\n";
  
  if( $user and $passwd ) {
    return undef unless $self->set_user( user => $user, password => $passwd );
  }
 
  return $self;
}

# intialise and load specific user information into the main object
sub set_user {
   my ($self, %args ) = @_;
   
   print "Package::Handler calling set_user()\n";
   
   $self->{_user} = new SOAP::User();
   print "Package::Handler created a blank SOAP::User object\n";

   unless ( ref($self) and $args{user} and 
            $self->{_user}->get_user($args{user})) {
            
      # user isn't know, return error string      
      undef $self->{_user};
      return "Could not load data for $args{user}";
   }
   
   # user data is loaded beforehand, so that the password is available
   # for testing. If the validation fails, user object is destroyed
   # before the error is sent, so that the called does not accidentially
   # get the user data.
   if( $args{password} ) {

      unless( $args{password} eq $self->{_user}->passwd()) {
      
         undef $self->{_user};
         return "Bad password for $args{user}";
      }
      
   } elsif( $args{cookie} ) {

      print "Package::Handler found a cookie in arguements\n";

      unless( $args{cookie} eq 
              make_cookie($args{user}, $self->{_user}->{passwd}) ) {
              
         undef $self->{_user};
         return "Authentication token for $args{user} invalid";
      }
      
      print "Package::Handler valided user using the cookie\n";
      
   } else {
   
      undef $self->{_user};
      return "No authentication present for $args{user}";
   
   } 
   
   return $self;             
}

# This is the code that is used to generate cookies based on the user
# name and password. It is NOT cryptographically sound, it is just a
# simple form of obfuscation, used as an example. Should be replaced
# before system goes live...
sub make_cookie {
   my ($user, $passwd) = @_;
   my $cookie = $user . "::" . md5_hex($passwd);
   $cookie =~ s/(.)/sprintf("%%%02x", ord($1))/ge;
   $cookie =~ s/%/%25/g;
   $cookie;
}

# ==========================================================================
# H A N D L E R S 
# ==========================================================================

# test function
sub ping {
   my $self = shift;
   
   print "Package::Handler ping()\n";
   
   # not callable as a static method, so must have a value
   # user object stored within             
   unless ( my $user = $self->{_user}) {
      return "The object is missing user data"
   }
   
   return SOAP::Data->name('return', 'ACK')->type('xsd:string');
} 

# test function
sub echo {
   my $self = shift;
   my @args = @_;
   
   print "Package::Handler echo(@args)\n";
   
   # not callable as a static method, so must have a value
   # user object stored within             
   unless ( my $user = $self->{_user}) {
      return "The object is missing user data"
   }
   
   return SOAP::Data->name('return', "ECHO @args")->type('xsd:string');
} 

              
1;                                
                  
                  
                  
                  
                  
                  
